#include "test1.h"

#include <string>

#include "helper.h"
#include "CoinPackedVector.hpp"
#include "CoinPackedMatrix.hpp"
#include "OsiSolverInterface.hpp"

bool test1_solve(OsiSolverInterface *s, const char *solvername)
{
	bool ret = true;
	char buffer[256];
	std::string test_name("test1");

	std::string log_path = std::string("test/") + solvername + "/" + test_name + ".log";
	FILE *log = fopen(log_path.c_str(), "w");

	logString(log, (test_name + "(" + solvername + ")").c_str());

	double inf = s->getInfinity();

	CoinPackedMatrix m;

	m.transpose();

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	m.appendRow(r0);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	m.appendRow(r1);

	int numcol = 2;

	double *obj = new double[numcol];
	obj[0] = 3;
	obj[1] = 1;

	double *collb = new double[numcol];
	collb[0] = 0;
	collb[1] = 0;

	double *colub = new double[numcol];
	colub[0] = inf;
	colub[1] = inf;

	int numrow = 2;

	double *rowlb = new double[numrow];
	rowlb[0] = -inf;
	rowlb[1] = -inf;

	double *rowub = new double[numrow];
	rowub[0] = 10;
	rowub[1] = 15;

	s->loadProblem(m, collb, colub, obj, rowlb, rowub);

	delete [] obj;
	delete [] collb;
	delete [] colub;

	delete [] rowlb;
	delete [] rowub;

	s->setObjSense(-1);

	bool optimal;

	s->resolve();
	optimal = s->isProvenOptimal();
	if (optimal) {
		sprintf(buffer, "%f %f", s->getColSolution()[0], s->getColSolution()[1]);
		logString(log, buffer);
		ret = ret && checkEquality(s->getColSolution()[0], 5);
		ret = ret && checkEquality(s->getColSolution()[1], 0);
	}
	else {
		fprintf(log, "no optimal solution.\n");
	}

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	optimal = s->isProvenOptimal();
	if (optimal) {
		sprintf(buffer, "%f %f", s->getColSolution()[0], s->getColSolution()[1]);
		logString(log, buffer);
		ret = ret && checkEquality(s->getColSolution()[0], 3);
		ret = ret && checkEquality(s->getColSolution()[1], 4);
	}
	else {
		fprintf(log, "no optimal solution.\n");
	}

	fclose(log);
	return ret;
}
