#include "test1.h"

#include "helper.h"
#include "CoinPackedVector.hpp"
#include "OsiSolverInterface.hpp"

bool test4_solve(OsiSolverInterface *s, const char *solvername)
{
	bool ret = true;
	char buffer[256];
	std::string test_name("test4");

	std::string log_path = std::string("test/") + solvername + "/" + test_name + ".log";
	FILE *log = fopen(log_path.c_str(), "w");

	logString(log, (test_name + "(" + solvername + ")").c_str());

	double inf = s->getInfinity();

	CoinPackedVector empty;

	s->addCol(empty, 0, inf, 3);
	s->addCol(empty, 0, inf, 1);

	CoinPackedVector r0;
	r0.insert(0, 2);
	r0.insert(1, 1);
	s->addRow(r0, -inf, 10);

	CoinPackedVector r1;
	r1.insert(0, 1);
	r1.insert(1, 3);
	s->addRow(r1, -inf, 15);

	s->setObjSense(-1);

	s->writeMps("test");

	bool optimal;

	s->resolve();
	optimal = s->isProvenOptimal();
	if (optimal) {
		sprintf(buffer, "%f %f", s->getColSolution()[0], s->getColSolution()[1]);
		logString(log, buffer);
		ret = ret && checkEquality(s->getColSolution()[0], 5);
		ret = ret && checkEquality(s->getColSolution()[1], 0);
	}
	else {
		fprintf(log, "no optimal solution.\n");
	}

	s->setObjCoeff(0, 1);
	s->setObjCoeff(1, 1);

	s->resolve();
	optimal = s->isProvenOptimal();
	if (optimal) {
		sprintf(buffer, "%f %f", s->getColSolution()[0], s->getColSolution()[1]);
		logString(log, buffer);
		ret = ret && checkEquality(s->getColSolution()[0], 3);
		ret = ret && checkEquality(s->getColSolution()[1], 4);
	}
	else {
		fprintf(log, "no optimal solution.\n");
	}

	fclose(log);
	return ret;
}
